#include <stdio.h>
#include <unistd.h>
#include <wiringPiI2C.h>
#include "uss.h"

// 測定の時間間隔[ms]
static const int t_uss = 50;

// 測定範囲の下限・上限[cm]
static const int uss_min =  16;
static const int uss_max = 600;

// センサからMIRS中心までの距離[cm]
static const int dist_center = 10;

static int fd_f, fd_l, fd_r, fd_b, fd_fl, fd_fr;

/*--------------------------------センサの初期化--------------------------------*/

//統括  一つでもエラーがでたら-1を返す
int uss_init(){
        int flag_f = 0, flag_l = 0, flag_r = 0, flag_b = 0, flag_fl = 0, flag_fr = 0;

        if(uss_init_f() != 0){
                printf(" front uss open faild !!\n");
                flag_f = -1;
        }
        if(uss_init_b() != 0){
                printf(" back uss open faild !!\n");
                flag_b = -1;
        }
        if(uss_init_l() != 0){
                printf(" left  uss open faild !!\n");
                flag_l = -1;
        }
        if(uss_init_r() != 0){
                printf(" right uss open faild !!\n");
                flag_r = -1;
        }
        if(uss_init_fl() != 0){
                printf(" front_left uss open faild !!\n");
                flag_fl = -1;
        }
        if(uss_init_fr() != 0){
                printf(" front_right uss open faild !!\n");
                flag_fr = -1;
        }


        if( flag_f == -1 || flag_l == -1 || flag_r == -1 || flag_b == -1 || flag_fl == -1 || flag_fr == -1) {
                return -1;
        }else{
                return 0;
        }
}

//正面
int uss_init_f(){
	fd_f = wiringPiI2CSetup(0x70);
	
	// ソフトウェアリビジョンの確認
	if(wiringPiI2CReadReg8(fd_f, 0x00) == 0x06){
		printf("USS_f opened\n");
		return 0;
	}else{
		printf("failed to open USS_F\n");
		return -1;
	}
}

//背面
int uss_init_b(){
	fd_b = wiringPiI2CSetup(0x72);
	
	// ソフトウェアリビジョンの確認
	if(wiringPiI2CReadReg8(fd_b, 0x00) == 0x06){
		printf("USS_B opened\n");
		return 0;
	}else{
		printf("failed to open USS_B\n");
		return -1;
	}
}

//左側
int uss_init_l(){
	fd_l = wiringPiI2CSetup(0x71);
	
	// ソフトウェアリビジョンの確認
	if(wiringPiI2CReadReg8(fd_l, 0x00) == 0x06){
		printf("USS_L opened\n");
		return 0;
	}else{
		printf("failed to open USS_L\n");
		return -1;
	}
}

//右側
int uss_init_r(){
	fd_r = wiringPiI2CSetup(0x73);
	
	// ソフトウェアリビジョンの確認
	if(wiringPiI2CReadReg8(fd_r, 0x00) == 0x06){
		printf("USS_R opened\n");
		return 0;
	}else{
		printf("failed to open USS_R\n");
		return -1;
	}
}

//左斜め前
int uss_init_fl(){
        fd_fl = wiringPiI2CSetup(0x74);

	//ソフトウェアリビジョンの確認
	if(wiringPiI2CReadReg8(fd_fl, 0x00) == 0x06){
                printf("USS_FL opened\n");
                return 0;
        }else{
                printf("failed to open USS_FL\n");
                return -1;
        }
}

//右斜め前
int uss_init_fr(){
        fd_fr = wiringPiI2CSetup(0x75);

        //ソフトウェアリビジョンの確認
        if(wiringPiI2CReadReg8(fd_fr, 0x00) == 0x06){
                printf("USS_FR opened\n");
                return 0;
        }else{
                printf("failed to open USS_FR\n");
                return -1;
        }
}


/*---------------------------センサの値取得---------------------------------*/

//統括 正面のセンサを1として時計回りに指定する
long uss_get(int a){
	long value;

	if(a == 1){
		value = uss_get_f();
	}else if(a == 2){
		value = uss_get_fr();
	}else if(a == 3){
		value = uss_get_r();
	}else if(a == 4){
		value = uss_get_b();
	}else if(a == 5){
		value = uss_get_l();
	}else if(a == 6){
		value = uss_get_fl();
	}else{
		value = -1;
	}

	return value;
}

//正面
long uss_get_f(){
	unsigned char lowbyte, highbyte;
	long val;
	
	wiringPiI2CWriteReg8(fd_f, 0x00, 0x51);
	usleep(t_uss * 1000);
	
	lowbyte  = wiringPiI2CReadReg8(fd_f, 0x03);
	highbyte = wiringPiI2CReadReg8(fd_f, 0x02);
	val = lowbyte + highbyte * 256;
	
	if(val < uss_min || val > uss_max){
		return -1;
	}else{
		return val;
	}
}

//背面
long uss_get_b(){
	unsigned char lowbyte, highbyte;
	long val;
	
	wiringPiI2CWriteReg8(fd_b, 0x00, 0x51);
	usleep(t_uss * 1000);
	
	lowbyte  = wiringPiI2CReadReg8(fd_b, 0x03);
	highbyte = wiringPiI2CReadReg8(fd_b, 0x02);
	val = lowbyte + highbyte * 256;
	
	if(val < uss_min || val > uss_max){
		return -1;
	}else{
		return val;
	}
}

//左側
long uss_get_l(){
	unsigned char lowbyte, highbyte;
	long val;
	
	wiringPiI2CWriteReg8(fd_l, 0x00, 0x51);
	usleep(t_uss * 1000);
	
	lowbyte  = wiringPiI2CReadReg8(fd_l, 0x03);
	highbyte = wiringPiI2CReadReg8(fd_l, 0x02);
	val = lowbyte + highbyte * 256;
	
	if(val < uss_min || val > uss_max){
		return -1;
	}else{
		return val;
	}
}

//右側
long uss_get_r(){
	unsigned char lowbyte, highbyte;
	long val;
	
	wiringPiI2CWriteReg8(fd_r, 0x00, 0x51);
	usleep(t_uss * 1000);
	
	lowbyte  = wiringPiI2CReadReg8(fd_r, 0x03);
	highbyte = wiringPiI2CReadReg8(fd_r, 0x02);
	val = lowbyte + highbyte * 256;
	
	if(val < uss_min || val > uss_max){
		return -1;
	}else{
		return val;
	}
}

//左斜め前
long uss_get_fl(){
        unsigned char lowbyte, highbyte;
        long val;

        wiringPiI2CWriteReg8(fd_fl, 0x00, 0x51);
        usleep(t_uss * 1000);

        lowbyte  = wiringPiI2CReadReg8(fd_fl, 0x03);
        highbyte = wiringPiI2CReadReg8(fd_fl, 0x02);
        val = lowbyte + highbyte * 256;

        if(val < uss_min || val > uss_max){
                return -1;
        }else{
                return val;
        }
}

//右斜め前
long uss_get_fr(){
        unsigned char lowbyte, highbyte;
        long val;

        wiringPiI2CWriteReg8(fd_fr, 0x00, 0x51);
        usleep(t_uss * 1000);

        lowbyte  = wiringPiI2CReadReg8(fd_fr, 0x03);
        highbyte = wiringPiI2CReadReg8(fd_fr, 0x02);
        val = lowbyte + highbyte * 256;

        if(val < uss_min || val > uss_max){
                return -1;
        }else{
                return val;
        }
}

